package org.coderu.core.impl.common.types;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;
import com.google.common.collect.Collections2;

public final class Package{
    
    public static final Function<Package,Collection<Package>> TO_CHILDS = 
            new Function<Package,Collection<Package>>(){
        
        public Collection<Package> apply(Package o) {
            return o.getChilds();
        }
        
    };

    public static final Function<Package,PackageName> TO_ID = new Function<Package,PackageName>(){
        public PackageName apply(Package o) {
            return o.getId();
        }
    };

    public static final Predicate<Package> IS_IMPL = createPredicateByName(PackageType.IMPL.getName());

    public static final Predicate<Package> IS_COMMON = createPredicateByName(PackageType.COMMON.getName());

    public static final Predicate<Package> IS_API = createPredicateByName(PackageType.API.getName());

    public static final Predicate<Package> IS_FACTORY = createPredicateByName(PackageType.FACTORY.getName());

    public static final Predicate<Package> IS_ROOT = new Predicate<Package>() {
        
        public boolean apply(Package o) {
            return o.getParent()==null;
        }
    };

    private static final Predicate<Package> createPredicateByName(final String name){
        return new Predicate<Package>(){

            public boolean apply(Package o) {
                return o.getLastName().equals(name);
            }
            
            @Override
            public String toString() {
            
                return String.format("is %s", name);
            }
            
        }; 
    }
    private final PackageName id;
    private final Collection<Package> childs;
    private Package parent;

    public Package(PackageName id) {
        this.id = id;
        this.childs = new LinkedList<Package>();
    }

    public Collection<String> getName() {
        return id.getName();
    }

    public Collection<Package> getChilds() {
        return childs;
    }

    public Collection<Package> getSiblings(){
        return parent != null ?
        Collections2.filter(parent.childs, Predicates.not(Predicates.equalTo(this))):
        Collections.<Package>emptyList();
    }
    
    public Package getParent() {
        return parent;
    }

    public PackageName getId() {
        return id;
    }

    public void setParent(Package parent) {
        this.parent = parent;
    }

    public String getLastName(){
        return id.getLastName();
    }
    
    @Override
    public int hashCode() {
        return new HashCodeBuilder(3,5).append(getName()).toHashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) { return false; }
        if (obj == this) { return true; }
        if (obj.getClass() != getClass()) {return false;}
        final Package rhs = (Package) obj;

        return new EqualsBuilder().append(id, rhs.id).isEquals();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this,ToStringStyle.SIMPLE_STYLE)
        .append(StringUtils.join(getName(),'.'))
        .toString();
    }

    public boolean isParentOf(Package packagge) {
        return id.isParentOf_(packagge.id);
    }
}