package org.coderu.core.impl.explorer.parser.impl;

import static com.google.common.base.Predicates.*;
import static org.coderu.common.utils.CollectionUtils.*;

import java.util.Collection;

import org.coderu.common.utils.FunctionFactory;
import org.coderu.core.api.Packagge;
import org.coderu.core.impl.common.types.PackageName;

import com.google.common.base.Function;
import com.google.common.base.Predicate;


public final class PackageFilterFactory {

    public static Predicate<Packagge> createFilter(
        final Collection<PackageName> components, final Collection<PackageName> excluded) {

        final Function<PackageName, Predicate<String>> toStrartWithPredicate = 
            FunctionFactory.concat( PackageName.TO_FULL_NAME, TO_START_WITH_PREDICATE);
        
        final Collection<Predicate<String>> packages = 
            transform(components, toStrartWithPredicate);
        
        final Collection<Predicate<String>> excludedPackages = 
            transform(excluded, toStrartWithPredicate);
        
        final Predicate<String> isOneOf = and(or(packages),not(or(excludedPackages)));
        
        return new Predicate<Packagge>(){
            public boolean apply(Packagge o) {
            
                return isOneOf.apply(o.getName());

            }
};
    }

    private static final Function<String,Predicate<String>> TO_START_WITH_PREDICATE = 
            new Function<String, Predicate<String>>() {
        
        public Predicate<String> apply(String o) {
            return startWith(o);
        }
    };
    
    private static final Predicate<String> startWith(final String prefix){
        return new Predicate<String>() {

            public boolean apply(String o) {

                return o.startsWith(prefix);
            }
        };
    }

    private PackageFilterFactory() {}
}
