package org.coderu.core.impl.explorer.parser.impl.loader;

import static org.coderu.common.utils.CollectionUtils.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import org.apache.commons.io.FilenameUtils;
import org.coderu.common.utils.ExceptionWrapper;
import org.coderu.core.api.Packagge;
import org.coderu.core.impl.explorer.parser.impl.loader.types.ByteCodeProvider;

import com.google.common.base.Function;
import com.google.common.base.Predicate;


public class JavaClassLoader {

    public JavaClassLoader(final Predicate<Packagge> paclkagePredicate){
    }
    
    public Collection<ByteCodeProvider> build(final Collection<File> classPath){
        final Collection<ByteCodeProvider> result = new LinkedList<ByteCodeProvider>();
        for(final File file: classPath){
            result.addAll( collect(file) );
        }
        return result;
    }

    private Collection<ByteCodeProvider> collect(final File file) {

        if(acceptJarFile(file)){
            final JarFile jarFile = createJarFile(file);
            return buildClasses(jarFile);
        }
        else if( file.isDirectory()){
            return buildClasses(new FileCollector(file).collect(), file);
        }
        else{
            throw new RuntimeException(String.format("unexpected file <%s>",file));
        }
    }

    private Collection<ByteCodeProvider> buildClasses(final Collection<File> collect, final File file) {

        return transform(collect, file2bytecodeprovider(file));
    }

    private static JarFile createJarFile(final File file) {

        return ExceptionWrapper.execute(new ExceptionWrapper.IFunctor<JarFile>(){
            public JarFile execute() throws Exception {
            return new JarFile(file);
        }});
    }
    
    private Collection<ByteCodeProvider> buildClasses(JarFile file) {

        final Collection<ByteCodeProvider> javaClasses = new ArrayList<ByteCodeProvider>();

        Enumeration<JarEntry> entries = file.entries();
        
        while (entries.hasMoreElements()) {
            ZipEntry e = (ZipEntry) entries.nextElement();
            if (FileCollector.isClassFile(e.getName())) {
                ByteCodeProvider jc = create(file,e);
                javaClasses.add(jc);
            }
        }

        return javaClasses;
    }
    
    private static final ByteCodeProvider create(final File file, final File base){
        
        final int basePathLength = FilenameUtils.normalizeNoEndSeparator(base.getPath()).length()+1;
        
        return new ByteCodeProvider() {

            public InputStream get() {

                return ExceptionWrapper.execute(new ExceptionWrapper.IFunctor<InputStream>(){
                    public InputStream execute() throws Exception {

                        return new FileInputStream(file);
                    }});
            }

            public String getName() {
                final String normalized = FilenameUtils.normalizeNoEndSeparator(file.getPath());
                return normalized.substring(basePathLength);
            }
            
            @Override
            public String toString() {
            
                final String normalized = FilenameUtils.normalizeNoEndSeparator(file.getPath());
                return normalized.substring(basePathLength);
            }
        };
    }

    private static Function<File,ByteCodeProvider> file2bytecodeprovider(final File base){
        return new Function<File, ByteCodeProvider>() {
            
            public ByteCodeProvider apply(File o) {
            
                return create(o, base);
            }
        };
    }
    private static final ByteCodeProvider create(final JarFile file, final ZipEntry e){
        return new ByteCodeProvider() {

            
            public InputStream get() {

                return ExceptionWrapper.execute(new ExceptionWrapper.IFunctor<InputStream>(){
                    public InputStream execute() throws Exception {

                        return file.getInputStream(e);
                    }});
            }

            public String getName() {
                return e.getName();
            }
            
            @Override
            public String toString() {
            
                return e.toString();
            }
        };
    }


    private static boolean acceptJarFile(File file) {
        return isJar(file) || isZip(file) || isWar(file);
    }

    private static boolean isWar(File file) {
        return existsWithExtension(file, ".war");
    }

    private static boolean isZip(File file) {
        return existsWithExtension(file, ".zip");
    }
 
    private static boolean isJar(File file) {
        return existsWithExtension(file, ".jar");
    }

    private static boolean existsWithExtension(File file, String extension) {
        return file.isFile() &&
            file.getName().toLowerCase().endsWith(extension);
    }

}
